@@ -59,10 +59,11 @@ class Hdf5MemoryMapped(DatasetMemoryMapped):
     snake_name = "hdf5"
     """Implements the vaex hdf5 file format"""
 
-    def __init__(self, path, write=False, fs_options={}, fs=None, nommap=None, group=None):
+    def __init__(self, path, write=False, fs_options={}, fs=None, nommap=None, group=None, _fingerprint=None):
         if nommap is None:
             nommap = not vaex.file.memory_mappable(path)
         super(Hdf5MemoryMapped, self).__init__(vaex.file.stringyfy(path), write=write, nommap=nommap, fs_options=fs_options, fs=fs)
+        self._cached_fingerprint = _fingerprint  # used during test, where we don't want to trigger an s3 call
         self._all_mmapped = True
         self._open(path)
         self.units = {}
